package abstractcollection;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2000
 * Company:      Joller-Voss GmbH
 * @author J.M.Joller
 * @version 1.0
 */

import java.util.*;

class FixedCol extends AbstractCollection {
    private Object[] array;
    int size = 0;

    FixedCol(int capacity) {
        array = new Object[capacity];
    }

    FixedCol() {
        this(10);  // Default capacity ist 10
    }

    FixedCol(Collection c) {
        int cSize = c.size();
        array = new Object[(cSize*110)/100];  // 10% fr Growth
        c.toArray(array);
        size = cSize;
    }

    public boolean add(Object e) {
        if (size == array.length)
            throw new IllegalStateException();
        array[size++] = e;
        return true;
    }

    public Iterator iterator() {
        // kein fail-fast iterator
        return new Iterator() {
            private int cursor = 0;
            private boolean removeProhibited = true;
            public boolean hasNext() {
                return cursor < size;
            }
            public Object next() {
                if (cursor==size)
                    throw new NoSuchElementException();
                removeProhibited = false;
                return array[cursor++];
            }
            public void remove() {
                if (removeProhibited)
                    throw new IllegalStateException();
                array[--cursor] = array[--size];
                array[size] = null;   // Eliminate "false reference"
                removeProhibited = true;
            }
        };
    }

    public int size() {
        return size;
    }
}
