package abstractlist;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2000
 * Company:      Joller-Voss GmbH
 * @author J.M.Joller
 * @version 1.0
 */

import java.util.*;

class FixedList extends AbstractList {
    int size;
    Object[] array;

    FixedList(int capacity) {
        array = new Object[capacity];
    }

    FixedList() {
        this(10);  // Default capacity ist 10
    }

    FixedList(Collection c) {
        int cSize = c.size();
        array = new Object[(cSize*110)/100];  //  10% fr Zuwachs
        c.toArray(array);
        size = cSize;
    }

    public void add(int index, Object e) {
        if (index < 0 || index > size) {
            throw new IndexOutOfBoundsException();
        }
        if (size==array.length) {
            throw new IllegalStateException();
        }
        System.arraycopy(array, index, array, index+1, size-index);
        array[index] = e;
        size++;
        modCount++;
    }
    public Object get(int index) {
        checkIndex(index);
        return array[index];
    }

    public Object remove(int index) {
        checkIndex(index);
        Object e = array[index];
        System.arraycopy(array, index+1, array, index, size-index-1);
        size--;
        array[size] = null;  // Eliminate "false reference"
        modCount++;
        return e;
    }

    // Diese Methode ist nicht zwingend ntig
    //
    protected void removeRange(int fromIndex, int toIndex) {
        //
        int numMoved = size - toIndex;
        System.arraycopy(array, toIndex, array, fromIndex, numMoved);

        //  Eliminiere "false references" um dem garbage collector zu helfen
        int newSize = size - (toIndex-fromIndex);
        while (size != newSize)
            array[--size] = null;
    }

    public Object set(int index, Object e) {
        checkIndex(index);
        Object o = array[index];
        array[index] = e;
        return o;
    }

    public int size() {
        return size;
    }

    // Utility Methode.
    private void checkIndex(int index) {
        if (index < 0 || index >= size) {
            throw new IndexOutOfBoundsException();
        }
    }
}
